@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 
-/* 
+/*
  * Copyright (C) 2002 Thomas Vander Stichele
  *
  * This program is free software; you can redistribute it and/or
@@ -26,7 +26,6 @@
 
 #include <config.h>
 
-//#include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gtk/gtk.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkliststore.h>
@@ -38,32 +37,36 @@
 #include <libgnomevfs/gnome-vfs-directory.h>
 #include <libgnome/gnome-macros.h>
 
-#include <gst/play/play.h>
 #include <gst/gconf/gconf.h>
 
 #include <string.h>
 
 #include "i18n-support.h"
-#include "audio-info.h"
+#include "../media-info/media-info.h"
 #include "audio-play.h"
 #include "audio-view.h"
 
+#define SCAN_LENGTH	5	/* FIXME: gconf key ? */
 /* list columns */
 enum {
 	FILE_COLUMN,
 	TYPE_COLUMN,
 	LENGTH_COLUMN,
-	//TITLE_COLUMN,
-	//ARTIST_COLUMN,
-	//BITRATE_COLUMN,
-	//TIME_COLUMN,
-	//SAMPLE_RATE_COLUMN,
-	COMMENT_COLUMN,
+	BITRATE_COLUMN,
+	METADATA_COLUMN,
 	PATH_URI_COLUMN,
 	AUDIO_INFO_COLUMN,
 	NUM_COLUMNS
 };
 
+typedef enum
+{
+  AUDIO_VIEW_INFO_STATE_NULL,
+  AUDIO_VIEW_INFO_STATE_START,
+  AUDIO_VIEW_INFO_STATE_IDLER,
+  AUDIO_VIEW_INFO_STATE_DONE
+} AudioViewInfoState;
+
 struct AudioView {
 	GtkWidget *widget;	/* top-level view widget */
 
@@ -76,12 +79,13 @@ struct AudioView {
 	GtkWidget *stop_button;
 	GtkWidget *play_button;
 	GtkWidget *next_button;
+	GtkWidget *scan_button;
 	GtkWidget *seek_scale;
 	/* GtkAdjustment *seek_adj; */
 	GtkWidget *time;
 
 	GtkWidget *status;
-	
+
 	GtkWidget *event_box;
 	char *location;
 	char *selection;	/* which track do you want to play ? */
@@ -89,19 +93,61 @@ struct AudioView {
 	gulong seek_changed_id;
 	gdouble last_seek_value; /* so we can see if we changed the slider
 				    or the user */
-
 	GList *audio_list;
 	AudioPlay *audio_play;
+	gboolean scanning;
+	guint scan_timeout;
+	GstMediaInfo *media_info;
 
+	/* error messages for later display */
+	gchar *error_message;
+
+	/* state information for idler */
+	AudioViewInfoState state;
+	GstMediaInfoStream *stream;	/* set by idler read */
+	GList *audio_item;	/* pointer to current audio_list item under
+				   inspection */
+	GtkTreeIter *iterp;
 };
 
 /* forward declarations */
-static void	audio_view_set_time 	(AudioView *view, 
+static void	audio_view_set_time	(AudioView *view,
 		                         gint seconds, gint length);
 
+/* debug functions */
+
+#ifdef DEBUG
+gboolean print_model_row (GtkTreeModel *model, GtkTreePath *path,
+		          GtkTreeIter *iter, gpointer data)
+{
+	GstMediaInfoStream *info;
+
+	gchar *name = gtk_tree_path_to_string (path);
+	g_print ("print_model_row: path %s\n", name);
+	g_free (name);
+	gtk_tree_model_get (model, iter,
+			    AUDIO_INFO_COLUMN, &info,
+			    -1);
+	g_print ("info: %p - ", info);
+	if (info)
+		g_print ("path: %s\n", info->path);
+	else
+		g_print ("NULL\n");
+}
+
+static void
+audio_view_tree_model_dump (AudioView *view)
+{
+	GtkTreeModel *model;
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (view->tree_view));
+	g_assert (GTK_IS_TREE_MODEL (model));
+	gtk_tree_model_foreach (model, print_model_row, NULL);
+}
+#endif
+
 /* helper functions */
 static GtkWidget *
-control_box_add_labeled_button (GtkWidget *control, 
+control_box_add_labeled_button (GtkWidget *control,
 		                const gchar *label, const gchar *file)
 {
 	GtkWidget *button;
@@ -110,11 +156,11 @@ control_box_add_labeled_button (GtkWidget *control,
 	gchar *location;
 
 	button = gtk_button_new ();
-	
+
 	location = g_strdup_printf ("%s/%s", PIXMAPSDIR, file);
 	pixbuf = gdk_pixbuf_new_from_file (location, NULL);
 	g_free (location);
-	if (pixbuf) 
+	if (pixbuf)
 	{
 		image = gtk_image_new_from_pixbuf (pixbuf);
 		gtk_container_add (GTK_CONTAINER (button), image);
@@ -127,6 +173,124 @@ control_box_add_labeled_button (GtkWidget *control,
 	return button;
 }
 
+/* check if the uri is playable and return the mime type if it is, NULL
+ * if not */
+char *
+audio_view_is_playable (const char *uri)
+{
+        char *type = NULL;
+
+        type = (char *) gnome_vfs_get_mime_type (uri);
+        /* FIXME: assuming we get an allocated copy and have ownership */
+        if (!type) return NULL;
+        if (
+            (strcmp (type, "application/x-ogg") != 0) &&
+            (strcmp (type, "audio/x-flac") != 0) &&
+            (strcmp (type, "audio/x-mp3") != 0) &&
+            (strcmp (type, "audio/x-wav") != 0)
+            )
+        {
+                g_free (type);
+                return NULL;
+        }
+        return type;
+}
+
+/* extract GList of string properties to a string of different lines
+ * this might not be very optimized but occurences of having multiple
+ * similar tags aren't that common to worry much about it */
+static gchar *
+audio_view_props_to_string (GList *list, const gchar *prepend)
+{
+	GList *p;
+	gchar *add;
+	gchar *temp;
+	gchar *result = NULL;
+
+	p = list;
+	while (p)
+	{
+		GstPropsEntry *entry = (GstPropsEntry *) p->data;
+		const gchar *val;
+		gst_props_entry_get_string (entry, &val);
+
+		add = g_strdup_printf ("%s%s\n", prepend, val);
+		if (result == NULL)
+		{
+			/* first pass */
+			result = add;
+		}
+		else
+		{
+			temp = g_strconcat (result, add, NULL);
+		        g_free (result);
+			g_free (add);
+			result = temp;
+		}
+		p = g_list_next (p);
+	}
+	return result;
+}
+
+/* extract the useful metadata to an allocated string */
+static gchar *
+audio_view_media_get_metadata (GstMediaInfoStream *info)
+{
+	gchar *metadata = NULL;
+	GList *artist = NULL;
+	GList *title = NULL;
+	gchar *artists;
+	gchar *titles;
+
+	GList *p;
+	GstMediaInfoTrack *track;
+
+	g_print ("DEBUG: getting metadata\n");
+	if (info->length_tracks < 1) return NULL;
+	if (info->tracks == NULL) return NULL;
+
+	track = info->tracks->data;
+	if (track == NULL) return NULL;
+	if (track->metadata == NULL) return NULL;
+	if (track->metadata->properties == NULL) return NULL;
+
+	p = track->metadata->properties->properties;
+
+	/* construct GLists of GstPropsEntries to artist and title */
+	while (p)
+	{
+		GstPropsEntry *entry = (GstPropsEntry *) p->data;
+		const gchar *name;
+		const gchar *val;
+		GstPropsType type;
+
+		name = gst_props_entry_get_name (entry);
+		type = gst_props_entry_get_type (entry);
+		if (g_ascii_strncasecmp (name, "artist", 6) == 0 &&
+		    type == GST_PROPS_STRING_TYPE)
+			artist = g_list_append (artist, entry);
+		if (g_ascii_strncasecmp (name, "title", 6) == 0 &&
+		    type == GST_PROPS_STRING_TYPE)
+			title = g_list_append (title, entry);
+		p = g_list_next (p);
+	}
+
+	/* now construct a string out of it */
+	artists = audio_view_props_to_string (artist, "Artist: ");
+	titles = audio_view_props_to_string (title, "Title: ");
+	if (artists && titles)
+	{
+		metadata = g_strconcat (artists, titles, NULL);
+		g_free (artists);
+		g_free (titles);
+	}
+	else if (artists) metadata = artists;
+	else metadata = titles;
+
+	/* we chomp because it looks better in the list view */
+	return g_strchomp (metadata);
+}
+
 /* set next track to play
  * return FALSE if this is the last track */
 static gboolean
@@ -137,14 +301,15 @@ audio_view_play_next (AudioView *view)
 	GtkTreeModel *model;
 	GtkTreeIter iter;
 	GtkTreePath *path;
-	AudioInfo *info;
+	GstMediaInfoStream *info;
 
 	g_assert (IS_AUDIO_PLAY (play));
+	g_print ("DEBUG: play_next: start\n");
 
-	audio_view_set_time (view, 0, 0);
 	sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (view->tree_view));
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (view->tree_view));
 	g_assert (GTK_IS_TREE_MODEL (model));
+	/* get selected into iter and try to get a next track in the model */
 	if (gtk_tree_selection_get_selected (sel, &model, &iter))
 	{
 		GtkTreePath *old_path;
@@ -154,18 +319,28 @@ audio_view_play_next (AudioView *view)
 		if (!gtk_tree_model_get_iter (model, &iter, path))
 		{
 			g_print ("DEBUG: play_next: already last\n");
-			gtk_tree_selection_unselect_path (sel, old_path);
+			//gtk_tree_selection_unselect_path (sel, old_path);
 			gtk_tree_path_free (path);
 			return FALSE;
 		}
 		gtk_tree_model_get (model, &iter,
 				    AUDIO_INFO_COLUMN, &info,
 				    -1);
+		g_print ("DEBUG: play_next: track to play is %s\n", info->path);
 	}
 	audio_view_set_playing (view, info, path);
 	gtk_tree_path_free (path);
+	return TRUE;
 }
 
+/* timeout for scanner */
+static gboolean
+scan_timeout (AudioView *view)
+{
+	audio_view_play_next (view);
+}
+
+
 /* ui functions */
 static void
 audio_view_set_time (AudioView *view, gint seconds, gint length)
@@ -194,17 +369,27 @@ audio_view_load_location (AudioView *view, const char *location)
 
   audio_view_update (view);
 }
-	
-/* set this track to play and update the treeview to the new path 
+
+/* set this track to play and update the treeview to the new path
  * if path is NULL, don't update it */
+/* FIXME: if info is NULL, that means we don't know yet.  Maybe
+ * force an update of that first before playing ? */
 static void
-audio_view_set_playing (AudioView *view, AudioInfo *info,
+audio_view_set_playing (AudioView *view, GstMediaInfoStream *info,
 		        GtkTreePath *path)
 {
 	gchar *message;
+	gchar *unescaped;
 	AudioPlay *play;
 	GError **error = NULL;
 
+	if (info == NULL)
+	{
+		message = g_strdup_printf ("ERROR: no information yet");
+		gtk_label_set_text (GTK_LABEL (view->status), message);
+		g_free (message);
+		return;
+	}
 	/* play it */
 	play = view->audio_play;
 	g_assert (IS_AUDIO_PLAY (play));
@@ -217,6 +402,28 @@ audio_view_set_playing (AudioView *view, AudioInfo *info,
 	g_print ("DEBUG: set_playing: setting %s to play\n", info->path);
 	audio_play_set_location (play, info->path, NULL);
 	audio_play_set_state (play, GST_STATE_PLAYING, error);
+	if (view->scanning)
+	{
+		guint64 nanosecs = audio_play_get_length (play);
+
+		if (view->scan_timeout >= 0)
+		{
+			g_source_remove (view->scan_timeout);
+			view->scan_timeout = -1;
+		}
+		g_print ("DEBUG: we're scanning, %lld\n", nanosecs);
+		if (nanosecs > SCAN_LENGTH * GST_SECOND)
+		{
+			nanosecs -= SCAN_LENGTH;
+			nanosecs /= 2;
+			g_print ("DEBUG: scanning mode, seek to somewhere in middle\n");
+			g_print ("DEBUG: seeking to %lld\n", nanosecs);
+			audio_play_seek_to_time (play, nanosecs);
+			view->scan_timeout = g_timeout_add (SCAN_LENGTH * 1000,
+					                    (GSourceFunc) scan_timeout,
+							    view);
+		}
+	}
 	if (error)
 	{
 		/* FIXME: handleme */
@@ -228,7 +435,7 @@ audio_view_set_playing (AudioView *view, AudioInfo *info,
 	}
 
 	/* status update */
-	message = g_strdup_printf ("Playing %s", info->file);
+	message = g_strdup_printf ("Playing %s", info->path);
 	gtk_label_set_text (GTK_LABEL (view->status), message);
 	g_free (message);
 
@@ -237,29 +444,55 @@ audio_view_set_playing (AudioView *view, AudioInfo *info,
 		gtk_tree_view_set_cursor (GTK_TREE_VIEW (view->tree_view),
 			                  path, NULL, FALSE);
 }
-	
-			    
+
 /* ui callbacks */
+void
+audio_view_error_handler (const gchar *message, gpointer data)
+{
+	GtkWindow *parent = GTK_WINDOW (data);
+	GtkWidget *dialog;
+
+	if (parent == NULL || !GTK_IS_WINDOW (parent))
+	{
+		/* no parent window, so not sure if ui windows will be shown */
+		g_warning (message);
+		/* save it for possible later display */
+		/* FIXME: remove this, we don't get at the view anyway */
+		/*
+		if (view->error_message) g_free (view->error_message);
+		view->error_message = g_strdup (message);
+		*/
+	}
+	dialog = gtk_message_dialog_new (parent, GTK_DIALOG_DESTROY_WITH_PARENT,
+			                 GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
+					 message);
+	/* destroy it if the dialog is run */
+	if (gtk_dialog_run (GTK_DIALOG (dialog)) != GTK_RESPONSE_NONE)
+		gtk_widget_destroy (dialog);
+}
+
 /* FIXME */
 static void
-row_activated_callback (GtkTreeView *tree_view, GtkTreePath *path, 
-                        GtkTreeViewColumn *column, 
+row_activated_callback (GtkTreeView *tree_view, GtkTreePath *path,
+                        GtkTreeViewColumn *column,
                         AudioView *view)
 {
 	GtkTreeIter iter;
 	GtkTreeModel *model = gtk_tree_view_get_model (tree_view);
-	AudioInfo *info;
+	GstMediaInfoStream *info;
+	gchar *pathname = gtk_tree_path_to_string (path);
+	g_print ("DEBUG: activated_cb: path %s\n", pathname);
+	g_free (pathname);
 
-	g_print ("You activated a row\n");
-	
 	if (! (gtk_tree_model_get_iter (model, &iter, path)))
 			return;
-	
+
 	gtk_tree_model_get (model, &iter,
 			    AUDIO_INFO_COLUMN, &info,
 			    -1);
+	g_print ("DEBUG: callback: get info %p (%s) playing (iterp %)\n",
+		 info, info->path, &iter);
 	audio_view_set_playing (view, info, NULL);
-	view->selection = g_strdup ("ENOCLUE");
 }
 
 static void
@@ -270,7 +503,7 @@ prev_activate (GtkButton *prev_button, AudioView *view)
 	GtkTreeModel *model;
 	GtkTreeIter iter;
 	GtkTreePath *path;
-	AudioInfo *info;
+	GstMediaInfoStream *info;
 
 	play = view->audio_play;
 	g_assert (IS_AUDIO_PLAY (play));
@@ -303,7 +536,7 @@ play_activate (GtkButton *play_button, AudioView *view)
 	AudioPlay *play;
 	gchar *path = NULL;
 	gchar *file;
-	AudioInfo *info;
+	GstMediaInfoStream *info;
 
 	/* try to get the selected track */
 	sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (view->tree_view));
@@ -314,15 +547,17 @@ play_activate (GtkButton *play_button, AudioView *view)
 		gtk_tree_model_get (model, &iter,
 				    AUDIO_INFO_COLUMN, &info,
 				    -1);
+		g_print ("DEBUG: pressed play on track %s\n", info->path);
 		audio_view_set_playing (view, info, NULL);
 	}
 	else
+	{
 		message = g_strdup_printf ("Make a selection first !");
-	gtk_label_set_text (GTK_LABEL (view->status),
-			    message);
-	g_free (message);
+		gtk_label_set_text (GTK_LABEL (view->status), message);
+		g_free (message);
+	}
 }
-	
+
 static void
 stop_activate (GtkButton *stop_button, AudioView *view)
 {
@@ -335,6 +570,14 @@ stop_activate (GtkButton *stop_button, AudioView *view)
 		g_print ("DEBUG: pressed stop, setting to ready\n");
 		audio_play_set_state (play, GST_STATE_READY, NULL);
 	}
+	/* remove callback if there */
+	if (view->scan_timeout >= 0)
+	{
+		g_source_remove (view->scan_timeout);
+		view->scan_timeout = -1;
+	}
+	gtk_range_set_value (GTK_RANGE (view->seek_scale),
+			     (double) 0.0);
 	audio_view_set_time (view, 0, 0);
 	gtk_label_set_text (GTK_LABEL (view->status),
 			    "Stopped.");
@@ -346,6 +589,33 @@ next_activate (GtkButton *next_button, AudioView *view)
 	audio_view_play_next (view);
 }
 
+static void
+scan_activate (GtkButton *scan_button, AudioView *view)
+{
+	g_print ("DEBUG: toggle scan\n");
+	view->scanning = !view->scanning;
+	g_print ("DEBUG: scanning %s\n", view->scanning ? "true" : "false");
+	/* FIXME: update toggle ? */
+	/* FIXME: act when playing or smth ? */
+	if (view->scanning)
+	{
+		AudioPlay *play = view->audio_play;
+		g_assert (IS_AUDIO_PLAY (play));
+		if (audio_play_get_state (play) == GST_STATE_PLAYING)
+		{
+			view->scan_timeout = g_timeout_add (SCAN_LENGTH * 1000,
+					            (GSourceFunc) scan_timeout,
+						    view);
+		}
+	}
+	else
+	{
+		/* remove callback if there */
+		if (view->scan_timeout >= 0)
+			g_source_remove (view->scan_timeout);
+		view->scan_timeout = -1;
+	}
+}
 /* GStreamer callbacks */
 static void
 have_tick_callback (AudioPlay *play, gint64 time_nanos, AudioView *view)
@@ -390,88 +660,143 @@ seek_changed_callback (GtkWidget *widget, AudioView *view)
 	gdouble value = gtk_range_get_value (GTK_RANGE (widget));
 	gdouble last_seek_value = view->last_seek_value;
 	g_assert (IS_AUDIO_PLAY (play));
+	if (audio_play_get_state (play) != GST_STATE_PLAYING) return;
 
-	//if (abs (last_seek_value - value) >= 1E-4)
-		/* user probably made the seek instead of us */
-		audio_play_seek_to_pos (play, value);
+	audio_play_seek_to_pos (play, value);
 	view->last_seek_value = value;
 }
 
-/* look up corresponding iter 
- * FIXME: this might get slow, browsing a list like this */
-static GtkTreeIter *
-audio_view_get_iter_from_info (AudioView *view, AudioInfo *info)
-{
-	GtkTreeIter iter, *ip;
-	AudioInfo *tree_info;
-	GtkTreeModel *model;
-	model = gtk_tree_view_get_model (GTK_TREE_VIEW (view->tree_view));
-
-	if (! gtk_tree_model_get_iter_first (model, &iter))
-		return NULL;
-
-	do
-	{
-		gtk_tree_model_get (model, &iter, 
-				    AUDIO_INFO_COLUMN, &tree_info, -1);
-		if (info == tree_info) return gtk_tree_iter_copy (&iter);
-	} while (gtk_tree_model_iter_next (model, &iter));
-	return NULL;
-}
+/* helper function to advance the media info getter to the next item
+ * if there is one */
+/* Returns: TRUE if there is a next track, FALSE otherwise */
 static gboolean
-audio_view_get_metadata_idler (AudioView *view)
+audio_view_media_info_next (AudioView *view)
+{
+	/* advance to the next audio item if there is one */
+	view->audio_item = view->audio_item->next;
+	/* FIXME: remove this debug function */
+	/* audio_view_tree_model_dump (view); */
+	if (view->audio_item)
+	{
+		view->state = AUDIO_VIEW_INFO_STATE_START;
+		return TRUE;
+	}
+	view->state = AUDIO_VIEW_INFO_STATE_DONE;
+	return FALSE;
+}
+
+static gboolean
+audio_view_get_media_info_idler (AudioView *view)
 {
 	GList *p;
-	AudioInfo *info;
 	GtkTreeIter iter;
-	
-	g_print ("DEBUG: running idler for view %p\n", view);
-   	for (p = view->audio_list; p != NULL; p = p->next) {
-		int min, sec, msec, remainder;
-		gchar *time;
-		GtkTreeIter *iterp;
-		info = (AudioInfo *) p->data;
-	
-		g_print ("getting metadata for info %p\n", info);
-		audio_info_update (info);
-		if (info->length > 0)
-		{
-			msec = info->length % 1000;
-			remainder = info->length / 1000;
-			sec = remainder % 60;
-			min = remainder / 60;
-			time = g_strdup_printf ("%d:%02d",
-					        min, sec);
-		}
-		else
-			time = g_strdup ("Unkown");
+	GstMediaInfoStream *info = NULL;
+	gchar *metadata;
 
-		/* fixme: get right position based on info instead of appd */
-		iterp = audio_view_get_iter_from_info (view, info);
-		g_print ("DEBUG: iter ptr %p\n", iterp);
-		gtk_list_store_set (view->list_store,
-				    iterp,
-				    LENGTH_COLUMN, time,
-				    //TITLE_COLUMN, g_strdup (info->title),
-				    //ARTIST_COLUMN, g_strdup (info->artist),
-				    //BITRATE_COLUMN, info->bitrate,
-				    //TIME_COLUMN, info->track_time,
-				    //SAMPLE_RATE_COLUMN, info->samprate,
-				    COMMENT_COLUMN, g_strdup (info->comment),
-				    //COMMENT_COLUMN, g_strdup_printf ("%p", info),
-				    AUDIO_INFO_COLUMN, info,
-				    -1);
-		g_free (time);
+	switch (view->state)
+	{
+		case AUDIO_VIEW_INFO_STATE_NULL:
+		{
+			g_print ("DEBUG: STATE is NULL\n");
+			view->audio_item = view->audio_list;
+			view->state = AUDIO_VIEW_INFO_STATE_START;
+			return TRUE;
+		}
+		case AUDIO_VIEW_INFO_STATE_START:
+		{
+			gchar *escaped_name;
+			gchar *path_uri;
+			gchar *uri;
+
+			view->iterp = (GtkTreeIter *) view->audio_item->data;
+			gtk_tree_model_get (GTK_TREE_MODEL (view->list_store),
+					    view->iterp, FILE_COLUMN, &uri, -1);
+			g_print ("DEBUG: get: view->iterp: %p (%s)\n",
+				 view->iterp, uri);
+
+			escaped_name = (char *) gnome_vfs_escape_string (uri);
+			path_uri = g_build_filename (view->location,
+						     uri, NULL);
+			g_free (escaped_name);
+			g_print ("getting media-info for path uri %s\n",
+			         path_uri);
+			gst_media_info_read_with_idler (view->media_info,
+							path_uri,
+							GST_MEDIA_INFO_ALL);
+			view->stream = NULL;
+			view->state = AUDIO_VIEW_INFO_STATE_IDLER;
+			return TRUE;
+		}
+		case AUDIO_VIEW_INFO_STATE_IDLER:
+		{
+			GstMediaInfoStream *info;
+			gchar *time;
+			gchar *bitrate;
+			gchar *metadata;
+
+			if ((view->stream) == NULL)
+			{
+				if (!gst_media_info_read_idler (view->media_info,
+								&(view->stream)))
+				{
+						g_warning ("error on media info idler !");
+						/* FIXME: better cleanup here */
+						audio_view_media_info_next (view);
+				}
+				return TRUE;
+			}
+			/* got info, update it */
+			info = view->stream;
+			if (info)
+			{
+				gint seconds = info->length_time / GST_SECOND;
+				gint sec = seconds % 60;
+				gint min = seconds / 60;
+
+				time = g_strdup_printf ("%d:%02d", min, sec);
+			        metadata = audio_view_media_get_metadata (info);
+				if (info->bitrate)
+					bitrate = g_strdup_printf ("%d kbs",
+						  info->bitrate / 1000);
+				else
+					bitrate = g_strdup_printf ("Unknown");
+			}
+			else
+			{
+				time = g_strdup ("Unkown");
+				metadata = g_strdup ("Unknown");
+			}
+			g_print ("DEBUG: set: view->iterp: %p (%s)\n",
+				 view->iterp, info->path);
+			gtk_list_store_set (view->list_store,
+					    view->iterp,
+					    LENGTH_COLUMN, time,
+					    BITRATE_COLUMN, bitrate,
+					    METADATA_COLUMN, metadata,
+					    TYPE_COLUMN, info->mime,
+					    AUDIO_INFO_COLUMN, info,
+					    -1);
+			view->stream = NULL;
+			g_free (time);
+			audio_view_media_info_next (view);
+			return TRUE;
+		}
+	case AUDIO_VIEW_INFO_STATE_DONE:
+			view->iterp = NULL;
+			view->audio_item = NULL;
+			view->state = AUDIO_VIEW_INFO_STATE_NULL;
+			return FALSE;
+		default:
+			g_warning ("Don't know what to do here\n");
+			return FALSE;
 	}
 	/* release song list */
 	/* FIXME: rather not free the audio info's here, we need them no ? */
 	//eel_g_list_free_deep_custom (song_list, (GFunc) audio_info_free, NULL);
-	return FALSE;
 }
 
 
-
-/* updates the view with info from the uri 
+/* updates the view with info from the uri
  * shows file names, then starts handler to get rest of info
  * to get a more responsive ui
  */
@@ -485,7 +810,7 @@ audio_view_update (AudioView *view)
 	GList *p;
 	GList *audio_list = NULL;
 	GList *attributes;
-	AudioInfo *info;
+	GstMediaInfoStream *info;
 	char *path_uri, *escaped_name;
 	char *path;
         gchar *message;
@@ -509,57 +834,48 @@ audio_view_update (AudioView *view)
 		return;
 	}
 
-	/* iterate through the directory, collecting playable audio files 
-	 * and FIXME: extracting metadata data if present */
+	/* iterate through the directory, collecting playable audio files
+	 * and FIXME: extracting media-info data if present */
+        /* populate the list with filenames */
+        gtk_list_store_clear (view->list_store);
 	for (node = list; node != NULL; node = node->next) {
-        	current_file_info = node->data;
+		current_file_info = node->data;
 
 	        /* skip invisible files, for now */
 	        if (current_file_info->name[0] == '.') continue;
 
                 escaped_name = (char *) gnome_vfs_escape_string (current_file_info->name);
-                path_uri = g_build_filename (view->location, 
+                path_uri = g_build_filename (view->location,
 				             current_file_info->name, NULL);
                 g_free (escaped_name);
 
+		/* FIXME: check if the mime type is playable */
 		/* we will get NULL if it's not playable */
-                info = audio_info_new (path_uri);
-		if (info)
+		if (audio_view_is_playable (path_uri))
 		{
 			/* add to list */
-	        	info->path = path_uri;
-			info->file = g_strdup (current_file_info->name);
-		        file_index += 1;
-			audio_list = g_list_prepend (audio_list, info);
+			gtk_list_store_append (view->list_store, &iter);
+			gtk_list_store_set (view->list_store, &iter,
+				            FILE_COLUMN,
+				            g_strdup (current_file_info->name),
+				            -1);
+			view->audio_list = g_list_append (view->audio_list,
+				                  gtk_tree_iter_copy (&iter));
 		}
-        }
+	}
         gnome_vfs_file_info_list_free (list);
 
-        audio_list = g_list_reverse (audio_list);
-
-        /* populate the list with filenames */
-        gtk_list_store_clear (view->list_store);
-        for (p = audio_list; p != NULL; p = p->next) {
-		info = (AudioInfo *) p->data;
-
-		gtk_list_store_append (view->list_store,
-				       &iter);
-		gtk_list_store_set (view->list_store,
-				    &iter,
-				    FILE_COLUMN, g_strdup (info->file),
-				    TYPE_COLUMN, g_strdup (info->type),
-				    AUDIO_INFO_COLUMN, info,
-				    -1);
-	}
-	view->audio_list = audio_list;
 	/* release song list */
 	/* FIXME: rather not free the audio info's here, we need them no ? */
 	/* so where do we move it to ? */
 	//eel_g_list_free_deep_custom (song_list, (GFunc) audio_info_free, NULL);
 
 	//g_free (uri);
-	//FIXME: add back metadata
-	//g_idle_add ((GSourceFunc) audio_view_get_metadata_idler, view);
+	gtk_widget_show (view->widget);
+	/* create a media info object */
+	view->media_info = gst_media_info_new (NULL);
+	g_print ("DEBUG: adding idler");
+	g_idle_add ((GSourceFunc) audio_view_get_media_info_idler, view);
 }
 
 /* set up the tree view based on the AudioView */
@@ -579,7 +895,7 @@ set_up_tree_view (AudioView *view)
 
         column = gtk_tree_view_column_new_with_attributes (_("File"),
                                                            cell,
-                                                           "text", 
+                                                           "text",
                                                            FILE_COLUMN,
 							   NULL);
         gtk_tree_view_column_set_sort_column_id (column, FILE_COLUMN);
@@ -591,10 +907,11 @@ set_up_tree_view (AudioView *view)
 	g_object_set (cell, "yalign", 0.0, NULL);
         column = gtk_tree_view_column_new_with_attributes (_("Type"),
                                                            cell,
-                                                           "text", 
+                                                           "text",
                                                            TYPE_COLUMN,
 							   NULL);
         gtk_tree_view_column_set_resizable (column, TRUE);
+        gtk_tree_view_column_set_sort_column_id (column, TYPE_COLUMN);
 	gtk_tree_view_append_column (tree_view, column);
 
         /* The length column */
@@ -602,22 +919,45 @@ set_up_tree_view (AudioView *view)
 	g_object_set (cell, "yalign", 0.0, NULL);
         column = gtk_tree_view_column_new_with_attributes (_("Length"),
                                                            cell,
-                                                           "text", 
+                                                           "text",
                                                            LENGTH_COLUMN,
 							   NULL);
         gtk_tree_view_column_set_resizable (column, TRUE);
+        gtk_tree_view_column_set_sort_column_id (column, LENGTH_COLUMN);
 	gtk_tree_view_append_column (tree_view, column);
 
-        /* The comment column */
-        column = gtk_tree_view_column_new_with_attributes (_("Comment"),
+        /* The bitrate column */
+        cell = gtk_cell_renderer_text_new ();
+	g_object_set (cell, "yalign", 0.0, NULL);
+        column = gtk_tree_view_column_new_with_attributes (_("Bitrate"),
                                                            cell,
-                                                           "text", 
-                                                           COMMENT_COLUMN,
+                                                           "text",
+                                                           BITRATE_COLUMN,
+							   NULL);
+        gtk_tree_view_column_set_resizable (column, TRUE);
+        gtk_tree_view_column_set_sort_column_id (column, BITRATE_COLUMN);
+	gtk_tree_view_append_column (tree_view, column);
+
+        /* The metadata column */
+        column = gtk_tree_view_column_new_with_attributes (_("Metadata"),
+                                                           cell,
+                                                           "text",
+                                                           METADATA_COLUMN,
 							   NULL);
         gtk_tree_view_column_set_resizable (column, TRUE);
 	gtk_tree_view_append_column (tree_view, column);
 }
+/* Returns: the error message if there was an error, or NULL if
+ * there wasn't */
+/* FIXME: this isn't really necessary to have anymore is it ?
+ */
+const gchar *
+audio_view_get_error (AudioView *view)
+{
+	return view->error_message;
+}
 
+/* Returns: the main widget created by the view */
 GtkWidget *
 audio_view_get_widget (AudioView *view)
 {
@@ -633,16 +973,21 @@ audio_view_new ()
 	GtkWidget *image;
 
 	view = g_new0 (AudioView, 1);
+	/* set to NULL what needs to be NULL */
+	view->error_message = NULL;
+
 	/* get a toplevel vbox */
 	view->widget = gtk_vbox_new (FALSE, 5);
-	
+
 	/* create list model */
+	/* FIXME: do we really need strings for int values ? */
 	view->list_store = gtk_list_store_new (
 		NUM_COLUMNS,
 		G_TYPE_STRING,	/* FILE_COLUMN */
 		G_TYPE_STRING,  /* TYPE_COLUMN */
 		G_TYPE_STRING,  /* LENGTH_COLUMN */
-		G_TYPE_STRING,  /* comment */
+		G_TYPE_STRING,  /* BITRATE_COLUMN */
+		G_TYPE_STRING,  /* METADATA_COLUMN */
 		G_TYPE_STRING,	/* path */
 		G_TYPE_POINTER	/* AUDIO_INFO_COLUMN */
 	);
@@ -660,11 +1005,11 @@ audio_view_new ()
 	/* get the scroll window */
 	view->scroll_window = gtk_scrolled_window_new (NULL, NULL);
         gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (view->scroll_window),
-                                        GTK_POLICY_AUTOMATIC, 
+                                        GTK_POLICY_AUTOMATIC,
 					GTK_POLICY_AUTOMATIC);
         gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (view->scroll_window),
                                              GTK_SHADOW_ETCHED_IN);
-        gtk_container_add (GTK_CONTAINER (view->scroll_window), 
+        gtk_container_add (GTK_CONTAINER (view->scroll_window),
                            view->tree_view);
 	gtk_box_pack_start (GTK_BOX (view->widget),
 			    view->scroll_window, TRUE, TRUE, 0);
@@ -688,23 +1033,29 @@ audio_view_new ()
 			view->control, "Play", "play.png");
 	g_signal_connect (G_OBJECT (view->play_button),
 			  "clicked", G_CALLBACK (play_activate), view);
-	
-	/* nex */
+
+	/* next */
 	view->next_button = control_box_add_labeled_button (
 			view->control, "Next", "next.png");
 	g_signal_connect (G_OBJECT (view->next_button),
 			  "clicked", G_CALLBACK (next_activate), view);
 
+	/* scan */
+	view->scan_button = control_box_add_labeled_button (
+			view->control, "Scan", "scan.png");
+	g_signal_connect (G_OBJECT (view->scan_button),
+			  "clicked", G_CALLBACK (scan_activate), view);
+
 	/* seek bar */
 	/*
-	view->seek_adj = gtk_adjustment_new (0.0, 0.0, 1.0, 
+	view->seek_adj = gtk_adjustment_new (0.0, 0.0, 1.0,
 			                              0.01, 0.1, 1);
 	view->seek_scale = gtk_hscale_new (view->seek_adj);
 						      */
 	view->seek_scale = gtk_hscale_new_with_range (0.0, 1.0, 0.001);
 	gtk_range_set_update_policy (GTK_RANGE (view->seek_scale),
 				     GTK_UPDATE_DELAYED);
-	view->seek_changed_id = g_signal_connect (G_OBJECT (view->seek_scale), 
+	view->seek_changed_id = g_signal_connect (G_OBJECT (view->seek_scale),
 			                          "value_changed",
 			  G_CALLBACK (seek_changed_callback), view);
 
@@ -732,9 +1083,24 @@ audio_view_new ()
 
 	/* create a play object */
 	view->audio_play = audio_play_new (&error);
-	g_assert (IS_AUDIO_PLAY (view->audio_play));
-	if (error) g_error ("got an error\n");
+	if (error)
+	{
+		/* no window yet, so no parent */
+		audio_view_error_handler (error->message, NULL);
+		g_error_free (error);
+	}
+	if (!IS_AUDIO_PLAY (view->audio_play))
+	{
+		g_warning ("Could not create audio_play object\n");
+		return NULL;
+	}
+
+	/* install the error handler */
+	audio_play_set_error_handler (view->audio_play,
+			              (AudioPlayErrorHandler) audio_view_error_handler,
+				      NULL);
 	audio_sink = gst_gconf_get_default_audio_sink ();
+	/* FIXME: do fallback instead of assert */
 	g_assert (GST_IS_ELEMENT (audio_sink));
 	audio_play_set_audio_sink (view->audio_play, audio_sink);
 	g_signal_connect (G_OBJECT (view->audio_play), "eos",
@@ -742,6 +1108,12 @@ audio_view_new ()
 	g_signal_connect (G_OBJECT (view->audio_play), "tick",
 			  G_CALLBACK (have_tick_callback), view);
 
+	view->media_info = NULL;
+	view->scanning = FALSE;
+	view->scan_timeout = -1; /* FIXME: what is the range for source ids */
+	/* state stuff */
+	view->state = AUDIO_VIEW_INFO_STATE_NULL;
+	view->audio_item = NULL;
 	return view;
 }
 
@@ -751,5 +1123,7 @@ audio_view_dispose (AudioView *view)
 	g_print ("DEBUG: audio_view_finalize\n");
 	/* no need to destroy widgets, gets done automatically on toplevel */
 	g_print ("DEBUG: audio_play: %p\n", view->audio_play);
-	audio_play_dispose (view->audio_play);
+	/* if it never got created we won't have it will we ? */
+	/* FIXME: maybe fix this in audio_play itself ? */
+	if (view->audio_play) audio_play_dispose (view->audio_play);
 }
